/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

import React, {useContext, useEffect, useState} from 'react';

import FrontendDataSetContext from '../../FrontendDataSetContext';
import DefaultRenderer from '../../data_renderers/DefaultRenderer';
import {
	getDataRendererById,
	getDataRendererByURL,
	getInputRendererById,
} from '../../utils/dataRenderers';
import DndTableCell from './dnd_table/Cell';

function InlineEditInputRenderer({
	itemId,
	options,
	rootPropertyName,
	type,
	value,
	valuePath,
	...otherProps
}) {
	const {itemsChanges, updateItem} = useContext(FrontendDataSetContext);

	const [InputRenderer, setInputRenderer] = useState(() =>
		getInputRendererById(type)
	);

	useEffect(() => {
		setInputRenderer(() => getInputRendererById(type));
	}, [type]);

	let inputValue = value;

	if (
		itemsChanges[itemId] &&
		typeof itemsChanges[itemId][rootPropertyName] !== 'undefined'
	) {
		inputValue = itemsChanges[itemId][rootPropertyName].value;
	}

	return (
		<InputRenderer
			{...otherProps}
			itemId={itemId}
			options={options}
			updateItem={(newValue) =>
				updateItem(itemId, rootPropertyName, valuePath, newValue)
			}
			value={inputValue}
		/>
	);
}

function TableCell({
	actions,
	inlineEditSettings,
	itemData,
	itemId,
	itemInlineChanges,
	options,
	rootPropertyName,
	value,
	valuePath,
	view,
}) {
	const {
		customDataRenderers,
		inlineEditingSettings,
		loadData,
		openSidePanel,
	} = useContext(FrontendDataSetContext);

	const [loading, setLoading] = useState(false);

	const contentRenderer = view.contentRenderer;

	let SyncDataRenderer = DefaultRenderer;

	if (contentRenderer) {
		if (customDataRenderers && customDataRenderers[contentRenderer]) {
			SyncDataRenderer = customDataRenderers[contentRenderer];
		}
		else {
			SyncDataRenderer = getDataRendererById(contentRenderer);
		}
	}

	if (view.contentRendererModuleURL) {
		SyncDataRenderer = null;
	}

	const [DataRenderer, setDataRenderer] = useState(() => SyncDataRenderer);

	useEffect(() => {
		if (!loading && view.contentRendererModuleURL && !DataRenderer) {
			setLoading(true);

			getDataRendererByURL(view.contentRendererModuleURL)
				.then((Component) => {
					setDataRenderer(() => Component);

					setLoading(false);
				})
				.catch(() => {
					setDataRenderer(() => null);

					setLoading(false);
				});
		}
	}, [view, loading, DataRenderer]);

	if (
		inlineEditSettings &&
		(itemInlineChanges || inlineEditingSettings?.alwaysOn)
	) {
		return (
			<DndTableCell columnName={String(options.fieldName)}>
				<InlineEditInputRenderer
					actions={actions}
					itemData={itemData}
					itemId={itemId}
					options={options}
					rootPropertyName={rootPropertyName}
					type={inlineEditSettings.type}
					value={value}
					valuePath={valuePath}
				/>
			</DndTableCell>
		);
	}

	return (
		<DndTableCell columnName={String(options.fieldName)}>
			{DataRenderer && !loading ? (
				<DataRenderer
					actions={actions}
					itemData={itemData}
					itemId={itemId}
					loadData={loadData}
					openSidePanel={openSidePanel}
					options={options}
					rootPropertyName={rootPropertyName}
					value={value}
					valuePath={valuePath}
				/>
			) : (
				<span
					aria-hidden="true"
					className="loading-animation loading-animation-sm"
				/>
			)}
		</DndTableCell>
	);
}

export default TableCell;
